@@ -14,7 +14,7 @@ use File::Glob qw(:bsd_glob);
 use POSIX qw(strftime);                                                         # http://www.cplusplus.com/reference/ctime/strftime/
 use Data::Dump qw(dump);
 
-our $VERSION = '2017.331';
+our $VERSION = '2017.429';
 
 &saveToS3('DataTableText') if 0;                                                # Save code
 
@@ -90,8 +90,7 @@ sub fileOutOfDate($@)
 #-------------------------------------------------------------------------------
 
 sub filePath(@)                                                                 # File
- {my (@file) = @_;
-  my $s = '';
+ {my (@file) = grep {$_} @_;                                                    # Drop empty path entries
   my $t = join '/', map {s/[\/\\]+\Z//r} @_;
   $t
  }
@@ -120,6 +119,44 @@ sub currentDirectory()
   $f.'/'
  }
 
+sub containingFolder($)                                                         # Path to folder containing a file
+ {my ($file) = @_;
+  return './' unless $file =~ m/\//;
+  my @w = split /\//, $file;
+  pop @w;
+  join '/', @w, ''
+ }
+
+sub xxx(@)                                                                      # Execute a command
+ {my $c = join ' ', @_;                                                         # Command to execute
+  say STDERR timeStamp, " ", $c;
+  my $r = qx($c 2>&1);
+  say STDERR $r;
+  $r
+ }
+
+sub findFiles($)                                                                # Find all the file under a folder
+ {my ($dir) = @_;                                                               # Folder to start the search with
+  my @f;
+  for(qx(find $dir))
+   {chomp;
+    next if -d $_;                                                              # Do not include folder names
+    push @f, $_;
+   }
+  @f
+ }
+
+sub findDirs($)                                                                 # Find all the folders under a folder
+ {my ($dir) = @_;                                                               # Folder to start the search with
+  my @d;
+  for(qx(find $dir))
+   {chomp;
+    next unless -d $_;                                                          # Include only folders
+    push @d, $_;
+   }
+  @d
+ }
+
 #-------------------------------------------------------------------------------
 # Make a path for a file name or a folder
 #-------------------------------------------------------------------------------
@@ -572,6 +609,18 @@ sub containingPowerOfTwo($)
    }
   undef
  }
+
+#-------------------------------------------------------------------------------
+# Extract package name from java file
+#-------------------------------------------------------------------------------
+
+sub javaPackage($)
+ {my ($javaFile) = @_;                                                                 # Number to check
+  my $s = readFile($javaFile);
+  my ($p) = $s =~ m(package\s+(\S+)\s*;);
+  $p
+ }
+
 #-------------------------------------------------------------------------------
 # Documentation
 #-------------------------------------------------------------------------------
@@ -658,19 +707,22 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
 @ISA          = qw(Exporter);
 @EXPORT       = qw(formatTable);
-@EXPORT_OK    = qw(appendFile checkKeys containingPowerOfTwo
-convertImageToJpx currentDirectory
+@EXPORT_OK    = qw(appendFile
+checkKeys containingPowerOfTwo
+containingFolder convertImageToJpx currentDirectory
 dateStamp dateTimeStamp
 extractDocumentation
 fileList fileModTime fileOutOfDate
-filePath filePathDir filePathExt fileSize formatTableBasic
+filePath filePathDir filePathExt fileSize findDirs findFiles formatTableBasic
 genLValueArrayMethods genLValueHashMethods
 genLValueScalarMethods genLValueScalarMethodsWithDefaultValues
 imageSize indentString
+javaPackage
 loadArrayArrayFromLines loadArrayFromLines
 loadHashArrayFromLines loadHashFromLines
 makePath nws pad powerOfTwo quoteFile
-readBinaryFile readFile saveToS3 timeStamp trim writeBinaryFile writeFile);
+readBinaryFile readFile saveToS3 timeStamp trim writeBinaryFile writeFile
+xxx);
 %EXPORT_TAGS  = (all=>[@EXPORT, @EXPORT_OK]);
 
 1;
@@ -781,12 +833,29 @@ modified under the same terms as Perl itself.
 =cut
 
 __DATA__
-use Test::More tests => 47;
+use Test::More tests => 57;
 if (1)                                                                          # File paths
  {ok filePath   (qw(/aaa bbb ccc ddd.eee)) eq "/aaa/bbb/ccc/ddd.eee";
   ok filePathDir(qw(/aaa bbb ccc ddd))     eq "/aaa/bbb/ccc/ddd/";
  }
 
+if (1)                                                                          # Unicode to local file
+ {use utf8;
+  my $z = "𝝰𝝱𝝲";
+  my $f = "$z.data";
+  unlink $f if -e $f;
+  ok !-e $f;
+  writeFile($f, $z);
+  ok  -e $f;
+  my $s = readFile($f);
+  ok $s eq $z;
+  ok length($s) == 3;
+  unlink $f;
+  ok !-e $f;
+  qx(rmdir $z) if -d $z;
+  ok !-d $z;
+ }
+
 if (1)                                                                          # Unicode
  {use utf8;
   my $z = "𝝰𝝱𝝲";
@@ -800,7 +869,7 @@ if (1)
   ok length($s) == 3;
   unlink $f;
   ok !-e $f;
-  print STDERR qx(rmdir $z);
+  qx(rmdir $z);
   ok !-d $z;
  }
 if (1)                                                                          # Binary
@@ -815,7 +884,7 @@ if (1)
   ok length($s) == 12;
   unlink $f;
   ok !-e $f;
-  print STDERR qx(rmdir $z);
+  qx(rmdir $z);
   ok !-d $z;
  }
 if (1)                                                                          # Format table and AA
@@ -942,3 +1011,17 @@ ok  pad('abc  ', 3).'=' eq "abc=";
 ok  pad('abc  ', 4).'=' eq "abc =";
 ok  pad('abc  ', 5).'=' eq "abc  =";
 ok  pad('abc  ', 6).'=' eq "abc   =";
+
+ok containingFolder("/home/phil/test.data") eq "/home/phil/";
+ok containingFolder("phil/test.data")       eq "phil/";
+ok containingFolder("test.data")            eq "./";
+
+if (1)
+ {my $f = "test.java";
+  writeFile("test.java", <<END);
+// Test
+package com.xyz;
+END
+  ok javaPackage($f) eq "com.xyz";
+  unlink $f;
+ }
